@@ -5,6 +5,7 @@
 # http://www.opensource.org/licenses/mit-license.php
 
 from smarkets.errors import Error as _Error
+from smarkets.streaming_api.eto import LogoutReason
 
 
 class ConnectionError(_Error):
@@ -47,3 +48,19 @@ class DownloadError(_Error):
 
     "Raised when a URL could not be fetched"
     pass
+
+
+class LoginError(_Error):
+
+    "Raised when a login is not successful"
+    def __init__(self, reason):
+        self.reason_msg = 'Unknown'
+        self.reason = reason
+        if reason in LogoutReason.values():
+            self.reason_msg = LogoutReason.Name(reason)
+
+
+class LoginTimeout(_Error):
+
+    "Raised when no message is received after sending login request"
+    pass
