@@ -1,15 +1,57 @@
 from __future__ import absolute_import
 
 import unittest
+from collections import namedtuple
 from itertools import chain, product
 
 import six
+
 from mock import patch
 from nose.tools import eq_
 from six.moves import xrange
 
 from smarkets import uuid
 from smarkets.streaming_api.api import eto, InvalidCallbackError, seto, StreamingAPIClient
+from smarkets.streaming_api.exceptions import LoginError, LoginTimeout
+from smarkets.streaming_api.framing import frame_decode_all
+
+SUCCESSFUL_LOGIN_RESPONSE_RAW = bytearray(
+    b'@\x08\x01\x12<\x08\x01\x10\x08\x18\x0024\n\x129RK4DpIK9HY5FpZKTo\x10\x02\x1a\x13hanson'
+    b'@smarkets.com \x93\xa7\xb1\xc5\xa9\xa6\xd2\x18\x14\x08$\x12\x04\x08\x02\x18\x00\xaa\x02'
+    b'\t\x08\xf4\x03\x10\xe8\x07\x18\xc8\x01\n\x08\x01\x12\x06\x08\x03\x10\x05\x18\x00\n\x08'
+    b'\x01\x12\x06\x08\x04\x10\x05\x18\x00')
+UNAUTHORIZED_LOGIN_RESPONSE_RAW = bytearray(
+    b'\x0e\x08\x01\x12\n\x08\x01\x10\t\x18\x00:\x02\x08\x07')
+PAYLOAD_THROTTLE_LIMITS_RAW = bytearray(
+    b'\x14\x08$\x12\x04\x08\x02\x18\x00\xaa\x02\t\x08\xf4\x03\x10\xe8\x07\x18\xc8\x01')
+LOGOUT_HEARTBEAT_TIMEOUT_RAW = bytearray(
+    b'\x0e\x08\x01\x12\n\x08\x08\x10\t\x18\x00:\x02\x08\x02')
+
+
+class Frame(namedtuple('Frame', 'bytes protobuf')):
+    pass
+
+
+def read_session_buff_gen(raw_response):
+    buffers = []
+    if raw_response:
+        decoded_response, dummy = frame_decode_all(raw_response)
+        for data in decoded_response:
+            payload = seto.Payload()
+            payload.ParseFromString(bytes(data))
+            buffers.extend([Frame(bytes=data, protobuf=payload)])
+
+    class indexer(object):
+        index = 0
+
+    def ret_buff():
+        if indexer.index < len(buffers):
+            val = buffers[indexer.index]
+            indexer.index += 1
+            return val
+        return None
+
+    return ret_buff
 
 
 class Handler(object):
@@ -37,6 +79,62 @@ class SmarketsTestCase(unittest.TestCase):
         self.mock_session = None
         self.client = None
 
+    def test_login_ok(self):
+        "Test the `Smarkets.login` method"
+        self.client.session.next_frame = read_session_buff_gen(SUCCESSFUL_LOGIN_RESPONSE_RAW)
+        self.client.login()
+        self.assertTrue(self.client.check_login())
+
+    def test_login_ok_async(self):
+        "Test the `Smarkets.login` method"
+        self.client.login(False)
+        self.client.session.next_frame = read_session_buff_gen(
+            SUCCESSFUL_LOGIN_RESPONSE_RAW +
+            PAYLOAD_THROTTLE_LIMITS_RAW +
+            PAYLOAD_THROTTLE_LIMITS_RAW)
+        self.client.read()
+        self.assertTrue(self.client.check_login())
+
+    def test_login_unauthorized(self):
+        "Test the `Smarkets.login` method for unauthorized"
+        self.client.session.next_frame = read_session_buff_gen(UNAUTHORIZED_LOGIN_RESPONSE_RAW)
+        try:
+            self.client.login()
+        except LoginError as ex:
+            self.mock_session.disconnect.assert_called_once()
+            self.assertEquals(eto.LOGOUT_UNAUTHORISED, ex.reason)
+            self.assertEquals(ex.reason_msg, 'LOGOUT_UNAUTHORISED')
+            return
+
+        assert False
+
+    def test_login_ok_then_timeout(self):
+        "Test the `Smarkets.login` method for a sequence of messages"
+        self.client.session.next_frame = read_session_buff_gen(
+            SUCCESSFUL_LOGIN_RESPONSE_RAW +
+            PAYLOAD_THROTTLE_LIMITS_RAW +
+            LOGOUT_HEARTBEAT_TIMEOUT_RAW)
+        try:
+            self.client.login()
+        except LoginError as ex:
+            self.mock_session.disconnect.assert_called_once()
+            self.assertEquals(eto.LOGOUT_HEARTBEAT_TIMEOUT, ex.reason)
+            self.assertEquals(ex.reason_msg, 'LOGOUT_HEARTBEAT_TIMEOUT')
+            return
+
+        assert False
+
+    def test_login_noresponse(self):
+        "Test the `Smarkets.login` when no login response has been received"
+        self.client.session.next_frame = read_session_buff_gen('')
+
+        try:
+            self.client.login()
+        except LoginTimeout:
+            return
+
+        assert False
+
     def test_logout_norecv(self):
         "Test the `Smarkets.logout` method"
         self.client.logout(False)
